<?php
namespace AustraliaPost\Extensions\Business\Settings;

use AustraliaPost\Extensions\Business\API\Wrapper\Connectors\My_Post_Connector;
use AustraliaPost\Extensions\Business\API\Wrapper\Label;
use AustraliaPost\Extensions\Business\Endpoints\Endpoints_Factory;
use AustraliaPost\Extensions\Business\Endpoints\Verify_Credentials_Endpoint;
use AustraliaPost\Extensions\Business\Endpoints\Verify_Merchant_Token_Endpoint;
use AustraliaPost\Extensions\Business\Helpers\Utilities;

class Settings {


	/**
	 * The single instance of the class.
	 *
	 * @var Settings
	 */
	protected static $_instance = null;

	/**
	 * @return Settings
	 */
	public static function get_instance()
	{
		if ( is_null( self::$_instance ) ) {
			self::$_instance = new self();
		}
		return self::$_instance;
	}

	/**
	 * Custom_Titles_Settings constructor.
	 */
	public function __construct() {
		add_action( 'admin_enqueue_scripts', [$this, 'enqueue_admin_scripts'] );
	}

	private static function get_payment_methods()
	{
		$payment_methods = ['stored' => 'Stored Payment'];

		$charging_accounts = get_option(My_Post_Connector::CHARGING_ACCOUNTS_OPTION_KEY, []);

		if (count($charging_accounts) === 0 ) {
			return $payment_methods;
		}

		foreach ($charging_accounts as $charging_account) {
			$payment_methods[$charging_account['account_number']] = sprintf('Charging Account (%s)', $charging_account['account_number']);
		}

		return $payment_methods;

	}


	public function enqueue_admin_scripts()
	{
		if (isset($_GET['section']) && $_GET['section'] === 'instance_auspost') {
			$verification_eparcel_nonce = Endpoints_Factory::get_endpoints_nonce(Verify_Credentials_Endpoint::class);
			$verification_mypost_nonce = Endpoints_Factory::get_endpoints_nonce(Verify_Merchant_Token_Endpoint::class);
			wp_enqueue_script('auspost-settings', plugin_dir_url( __FILE__ ) . 'assets/settings.js', [], '1.0', true);
			wp_localize_script('auspost-settings', 'verification_nonce', ['eparcel' => $verification_eparcel_nonce, 'mypost' => $verification_mypost_nonce]);
		}
	}

	/**
	 * @return array
	 */
	public static function settings()
	{
		$settings = [];

		$settings['title_api_credentials'] = [
			'title'       => __( 'Account<hr>', 'woocommerce-australia-post-pro' ),
			'type'        => 'title',
			'description' => '',
		];

		$settings['account_type'] = [
			'title' => __('Account Type', 'woocommerce-australia-post-pro'),
			'type' => 'select',
			'options' => [
				'mypost' => 'MyPost Business',
				'eparcel' => 'eParcel',
			],
			'default' => 'eparcel',
			'description' => __('Choose your business account type with Australia Post.', 'woocommerce-australia-post-pro'),
		];

		$settings['enabled_contract_mode'] = [
			'title' => __('Enable Contract Postage', 'woocommerce-australia-post-pro'),
			'type' => 'checkbox',
			'label' => __('Enable ', 'woocommerce-australia-post-pro'),
			'default' => 'no',
			'description' => __('Show contracted prices at the Checkout page. <i>You do not need to enable this option in order to print labels.</i>', 'woocommerce-australia-post-pro'),
		];


		$settings['label_layout'] = [
			'title' => __('Label Layout', 'woocommerce-australia-post-pro'),
			'type' => 'select',
			'options' => Label::available_layouts(),
			'description' => __('Choose the layout of the generated labels.', 'woocommerce-australia-post-pro'),
		];

		$settings['label_branded'] = [
			'title' => __('Branded Labels', 'woocommerce-australia-post-pro'),
			'type' => 'checkbox',
			'label' => __('Enable ', 'woocommerce-australia-post-pro'),
			'default' => 'yes',
			'description' => __('Add Australia Post branding to the label.', 'woocommerce-australia-post-pro'),
		];

		$settings['add_instructions_to_labels'] = [
			'title' => __('Add Delivery Instructions to Labels', 'woocommerce-australia-post-pro'),
			'type' => 'checkbox',
			'label' => __('Enable ', 'woocommerce-australia-post-pro'),
			'default' => 'yes',
			'description' => __('If you enabled this option, the customer provided delivery instructions will be added to the printed label. The text is limited to 128 characters.', 'woocommerce-australia-post-pro'),
		];

		$accountType = null;
		if (isset($_POST['woocommerce_instance_auspost_account_type'])) {
			$accountType = $_POST['woocommerce_instance_auspost_account_type'];
		}

		$postage_products = Utilities::get_postage_products(false, $accountType);

		if ($postage_products) {
			$settings['enabled_contract_products'] = [
				'title' => __('Enabled Contract Options', 'woocommerce-australia-post-pro'),
				'type' => 'multiselect',
				'class' => 'availability wc-enhanced-select',
				'default' => array_keys($postage_products),
				'options' => $postage_products,
			];
		}

		$settings['api_merchant_token'] = [
			'title' => __('Merchant Token', 'woocommerce-australia-post-pro'),
			'type' => 'password',
			'description' => __(sprintf('Please visit <a href="%s" target="_blank">how to obtain your Merchant Token from your MyPost Business account.</a>', 'https://wpruby.com/knowledgebase/labels-pro-how-to-connect-the-plugin-with-your-mypost-business-account/'), 'woocommerce-australia-post-pro'). self::get_verify_button('mypost'),
		];

		if (Utilities::isMyPost() && self::get_payment_methods() > 1) {
			$settings['payment_method'] = [
				'title'     => __('Labels Payment Method', 'woocommerce-australia-post-pro'),
				'type'      => 'select',
				'default'   => 'stored',
				'options'   => self::get_payment_methods(),
			];
		}


		$settings['api_account_number'] = [
			'title' => __('Account Number', 'woocommerce-australia-post-pro'),
			'type' => 'text',
			'description' => __('Your Australia Post business account number.
			', 'woocommerce-australia-post-pro'),
		];
		$settings['api_password'] = [
			'title' => __('API Password', 'woocommerce-australia-post-pro'),
			'type' => 'password',
			'description' => __('The API Password should have been sent to you by Australia Post along with the API Key.', 'woocommerce-australia-post-pro') . self::get_verify_button('eparcel'),
		];

		return $settings;
	}

	private static function get_verify_button($type = 'eparcel') {

		if($type === 'eparcel') {
			return '
			<br>
			You can verify the credentials now by clicking on Verify. Make sure that Account Number, Password and API key are valid.
			<button id="btn-verify-credentials" type="button" class="button button-secondary">Verify</button>';
		} else {
			return '
			<br>
			You must verify the merchant token now by clicking on Verify. Without verifiying, you will not be able to connect your MyPost Business account.
			<button id="btn-verify-merchant-token" type="button" class="button button-secondary">Verify</button>
		';
		}

	}

}
